#######################################
#####  M0 #####
#######################################

likM0=function(N,n,sx,S) {
  p=sx/(N*S)
  lfactorial(N)-lfactorial(N-n)+sx*log(p)+(S*N-sx)*log(1-p)
}

LciM0=function(N,p,S,Nupp=N*10) {
  n=N*(1-dbinom(0,S,p))
  sx=N*p*S
  center=likM0(N,n,sx,S)
  Nlfun=try(uniroot(function(Nl) 2*(likM0(Nl,n,sx,S)-center)+(1.96^2),c(n,N)))
  if(mode(Nlfun)!="character"){
    Nl=Nlfun$root
  }else{
    Nl=n
  }
  Nu=uniroot(function(Nu) 2*(likM0(Nu,n,sx,S)-center)+(1.96^2),c(N,Nupp))$root
  (Nu-Nl)/N
}

########################################
#####  Mb #####
#######################################



likMb=function(N,n,n0p,S) {
  p=n/(n0p+n+S*(N-n))
  lfactorial(N)-lfactorial(N-n)+n*log(p)+(S*(N-n)+n0p)*log(1-p)
}

### EXPECTED VALUE and VARIACE of R (R=n.0p/M)

E.R_V.R=function(N,p,S){

dd=c()
for(i in 1:S){
	
	temp=(p*(1-p)^(i-1))/(1-(1-p)^S)*(i-1)
	
	dd=c(dd,temp)
	
}

EV.R=sum(dd)
names(EV.R)="Expected R"



dd=c()
for(i in 1:S){
		
	temp=(((i-1)-EV.R)^2*((p*(1-p)^(i-1))/(1-(1-p)^S)))/(N*(1-(1-p)^S))
	
	dd=c(dd,temp)
}	

VAR.R=sum(dd)
names(VAR.R)="Varianza di R"

return(c(EV.R,VAR.R))

}

p.fail=function(N,p,S) {
R.EV=E.R_V.R(N,p,S)
R.threshold=(S-1)/2
E.failure=pnorm((R.threshold-R.EV[1])/sqrt(R.EV[2]),lower.tail=FALSE)
names(E.failure)=NULL
E.failure
}


LciMb=function(N,p,S,Nupp=N*10) {
  n=N*(1-dbinom(0,S,p))
  sx=N*p*S
#  aa=p*(1-p)^(1:(S-1))*(1:(S-1))

  aa=c()
  for(i in 1:S){
    aa[i]=(i-1)*(p*(1-p)^(i-1))/(1-dbinom(0,S,p))
  }

  n0p=n*sum(aa)
  
  n=round(n)
  n0p=round(n0p)
  
  center=likMb(N,n,n0p,S)
  
  Nlfun=try(uniroot(function(Nl) 2*(likMb(Nl,n,n0p,S)-center)+(1.96^2),c(n,N)))
  if(mode(Nlfun)!="character"){
    Nl=Nlfun$root
  }else{
    Nl=n
  }

  Nufun=try(uniroot(function(Nu) 2*(likMb(Nu,n,n0p,S)-center)+(1.96^2),c(N,Nupp)))
  if(mode(Nufun)!="character"){
    Nu=Nufun$root
  }else{
    Nu=Nupp
  }

  Nu=round(Nu)
  Nl=round(Nl)

  lci=(Nu-Nl)/N

output=list(Lci=lci,CI=c(Nl,Nu),N=N,p=p,En=n,En0p=n0p)

return(output)
}

####################################################################
### Mh-betabbinomial ###
####################################################################

## assume beta<alp,bet> mixing distribution 

lik.Mhbb=function(N,f,n,alp,bet,S) {
p=choose(S,0:S)*gamma(alp+bet)*gamma(alp+0:S)*gamma(bet+S-0:S)
p=p/(gamma(alp)*gamma(bet)*gamma(alp+bet+S))
lfactorial(N)-lfactorial(N-n)-lfactorial(n)+(N-n)*log(p[1])+sum(f[-1]*log(p[-1]))
}

## int (1-x)^S dbeta(x,alp,bet) = (Gamma(a+b)Gamma(b+S)/Gamma(a+b+S)Gamma(b))

LciMhbb2=function(N,S,alp,bet,Nupp=N*10) {
n=N*(1-(gamma(alp+bet)*gamma(bet+S))/(gamma(alp+bet+S)*gamma(bet)))
p=choose(S,0:S)*gamma(alp+bet)*gamma(alp+0:S)*gamma(bet+S-0:S)
p=p/(gamma(alp)*gamma(bet)*gamma(alp+bet+S))
f=round(N*p)

  z2=qnorm(0.975)^2

log.lik=-sapply(n:Nupp, function(j) optim(log(c(alp,bet)), function(x) -lik.Mhbb(j,f,n,exp(x[1]),exp(x[2]),S))$value)
Nrange=n:Nupp

  N.hat=Nrange[which(log.lik==max(log.lik))]
  
  Nu=N.hat
  
  if (N.hat <= Nupp) {
        i = 1
        while (2 * (max(log.lik) - log.lik[which(log.lik == 
            (max(log.lik))) + i]) < z2 & (N.hat + i)<Nupp)
            i = i + 1
        }
        Nu = N.hat + i-1
        
   Nl=N.hat
    if (N.hat > n) {
        i = 1
        while (2 * (max(log.lik) - log.lik[(which(log.lik == 
            (max(log.lik))) - i)]) < z2 & (N.hat - i)>n)            			i = i + 1
        }
        Nl = N.hat - (i - 1) 

   return((Nu-Nl)/N)
}

LciMhbb=function(N,S,alp,bet,Nupp=N*10) {
n=N*(1-(gamma(alp+bet)*gamma(bet+S))/(gamma(alp+bet+S)*gamma(bet)))
p=choose(S,0:S)*gamma(alp+bet)*gamma(alp+0:S)*gamma(bet+S-0:S)
p=p/(gamma(alp)*gamma(bet)*gamma(alp+bet+S))
f=round(N*p)

  z2=qnorm(0.975)^2

  log.lik=lik.Mhbb(n:Nupp,f,n,alp,bet,S)
  Nrange=n:Nupp

  N.hat=Nrange[which(log.lik==max(log.lik))]
  
  Nu=N.hat
  
  if (N.hat <= Nupp) {
        i = 1
        while (2 * (max(log.lik) - log.lik[which(log.lik == 
            (max(log.lik))) + i]) < z2 & (N.hat + i)<Nupp)
            i = i + 1
        }
        Nu = N.hat + i-1
        
   Nl=N.hat
    if (N.hat > n) {
        i = 1
        while (2 * (max(log.lik) - log.lik[(which(log.lik == 
            (max(log.lik))) - i)]) < z2 & (N.hat - i)>n)            			i = i + 1
        }
        Nl = N.hat - (i - 1) 

   return((Nu-Nl)/N)
}

#######################################
##### 2 par Mt  #####
#######################################

likMt2=function(N,n,n2,S) {
  p=rep(n2/(N),S/2)
  lfactorial(N)-lfactorial(N-n)+sum(nS*log(p)+(N-nS)*log(1-p))
}


LciMt2=function(N,p,S,Nupp=N*10) {
  p=rep(p,S/2)
  n=N*(1-prod(1-p))
  nS=N*p
  center=likMt(N,n,nS)
  Nlfun=try(uniroot(function(Nl) 2*(likMt(Nl,n,nS)-center)+(1.96^2),c(n,N)))
  if(mode(Nlfun)!="character"){
    Nl=Nlfun$root
  }else{
    Nl=n
  }
  Nu=uniroot(function(Nu) 2*(likMt(Nu,n,nS)-center)+(1.96^2),c(N,Nupp))$root
  (Nu-Nl)/N
}
#######################################
##### general Mt #####
#######################################

likMt=function(N,n,nS) {
  p=nS/(N)
  lfactorial(N)-lfactorial(N-n)+sum(nS*log(p)+(N-nS)*log(1-p))
}


LciMt=function(N,p,Nupp=N*10) {
  n=N*(1-prod(1-p))
  nS=N*p
  center=likMt(N,n,nS)
  Nlfun=try(uniroot(function(Nl) 2*(likMt(Nl,n,nS)-center)+(1.96^2),c(n,N)))
  if(mode(Nlfun)!="character"){
    Nl=Nlfun$root
  }else{
    Nl=n
  }
  Nu=uniroot(function(Nu) 2*(likMt(Nu,n,nS)-center)+(1.96^2),c(N,Nupp))$root
  (Nu-Nl)/N
}
